﻿' Velleman K8055 Voltage Datalogger
'       Copyright (c) 2012  Vanderhaegen Bart
'
'   Version 2.0
'
'   This software will use the K8055 card as a voltage logger: the values of the A/D
'   channels will be read periodically and the data will be written to a .CSV file.
'
'   THIS SOFTWARE IS A BETA VERSION! It comes without any warranties.
'
'
Imports System.IO


Public Class frmMain


#Region "Declarations"
    ' Declaration of the system wide variables
    Dim REQ_SAMPLES As Integer = 4
    Dim avg1 As Integer                         ' averaging help variable 1
    Dim avg2 As Integer                         ' averaging help variable 2
    Dim avgcounter As Integer                   ' averaging counter

    ' Declaration of the datalogging related variables
    Dim dataloggerStartTime As DateTime         ' the start time of the datalog (in ticks)
    Dim lineCount As Integer = 0                ' the datalogger size
    Dim dataloggerRunning As Boolean = False    ' the state of the datalogger
    Dim dataloggerFile As String                ' the location of the datalogger file
    Dim textToInsert As String = ""             ' the text to be inserted during the next cycle
    Dim startTicks As Int64                     ' the start time of the datalog (in ticks)

    ' Define the streamwriter object: the datastream of the datalogger file
    Dim sw As StreamWriter

    ' Define the parameters objects for the A/D channels
    Dim chData(2) As channelData

    ' Define the K8055 object
    Dim myK8055 As New K8055(False)
#End Region




#Region "Setting Management"
    ''' <summary>
    ''' This sub will load all of the saved program parameters.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub loadSettings()
        With My.Settings
            ' Channel 1
            chData(0).enable = .ch1Enable
            chData(0).name = .ch1Text
            chData(0).attenuationMode = .ch1AttenuationMode
            chData(0).Rext1 = .ch1Rext1
            chData(0).Rext2 = .ch1Rext2
            chData(0).bypassResistor = .ch1ByPassResistor
            groupCH1.Text = "CH 1 - " & chData(0).name

            ' Channel 2
            chData(1).enable = .ch2Enable
            chData(1).name = .ch2Text
            chData(1).attenuationMode = .ch2AttenuationMode
            chData(1).Rext1 = .ch2Rext1
            chData(1).Rext2 = .ch2Rext2
            chData(1).bypassResistor = .ch2ByPassResistor
            groupCH2.Text = "CH 2 - " & chData(1).name

            ' Basic Information
            chkSK5.Checked = .jumperSK5
            chkSK6.Checked = .jumperSK6
            cmbOptions.SelectedIndex = .selectedAverageCount
        End With
    End Sub




    ''' <summary>
    ''' This sub will save all of the program parameters to the system for later retrieval.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub saveSettings()
        With My.Settings
            ' Channel 1
            .ch1Enable = chData(0).enable
            .ch1Text = chData(0).name
            .ch1AttenuationMode = chData(0).attenuationMode
            .ch1Rext1 = chData(0).Rext1
            .ch1Rext2 = chData(0).Rext2
            .ch1ByPassResistor = chData(0).bypassResistor

            ' Channel 2
            .ch2Enable = chData(1).enable
            .ch2AttenuationMode = chData(1).attenuationMode
            .ch2Text = chData(1).name
            .ch2Rext1 = chData(1).Rext1
            .ch2Rext2 = chData(1).Rext2
            .ch2ByPassResistor = chData(1).bypassResistor

            ' Basic Information
            .jumperSK5 = chkSK5.Checked
            .jumperSK6 = chkSK6.Checked
            .selectedAverageCount = cmbOptions.SelectedIndex
        End With

        My.Settings.Save()
    End Sub
#End Region




#Region "Form Events"
    ''' <summary>
    ''' frmMain_FormClosing is executed when the form closes. The function will cancel itself when
    ''' the voltagelogger is currently recording. Otherwise it will allow the program to quit, while
    ''' saving all of the settings first.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If dataloggerRunning Then
            ' Show an error when the datalogger is running: the program can't be stopped in the mean time.
            messageBox("The datalogger is running: you cannot stop the program")
            e.Cancel = True
            Exit Sub
        Else
            saveSettings()
        End If
    End Sub



    ''' <summary>
    ''' frm_Load contains all the procedures that have to be executed during the loading of the program.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub frm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Load the channel data objects
        chData(0) = New channelData()
        chData(1) = New channelData()

        ' Load the combo box with the possible events
        ' Attention: the listIndex property is used to retrieve the selected value.
        With cmbOptions
            For i As Integer = 1 To 10
                .Items.Add("average " & Format(i, "00") & " samples")
            Next
            .SelectedIndex = 9
        End With

        ' Setup the timer object
        With tmrBgReader
            .Enabled = False
            .Interval = 50
        End With

        ' Set the initial color of the shape
        shape.FillColor = Color.Gray

        ' Load the settings from the memory
        loadSettings()

        lblLineCount.Text = ""
        lblDuration.Text = ""
        chData(0).voltsMin = -1
        chData(1).voltsMin = -1
    End Sub
#End Region



#Region "Logging Code"
    ''' <summary>
    ''' sampleChannel will read a value from the K8055 card. This value will be added to the 
    ''' average counter.
    ''' </summary>
    ''' <param name="ch">Channel that is to be averaged</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function sampleChannel(ByVal ch As Integer) As Boolean
        ch = ch - 1 ' decrease the channel with one to match the zero-based index of the ChData array.

        ' Check if the selected channel is inactive. If so, exit the function.
        If chData(ch).enable = False Then Return False

        Dim rawADvalue As Integer = myK8055.ReadAnalogInput(ch + 1)
        chData(ch).rawAVGtotal = chData(ch).rawAVGtotal + rawADvalue
        chData(ch).AVGcounter += 1
        Return chData(ch).AVGcounter > REQ_SAMPLES
    End Function



    ''' <summary>
    ''' calculateStatistics will calculate the statistics for the given channel
    ''' </summary>
    ''' <param name="ch"></param>
    ''' <remarks></remarks>
    Public Sub calculateStatistics(ByVal ch As Integer)
        ch = ch - 1 ' decrease the channel with one to match the zero-based index of the ChData array.

        Dim rawAD As Integer = Int(chData(ch).rawAVGtotal / chData(ch).AVGcounter)
        If rawAD <> (chData(ch).rawAVGtotal / chData(ch).AVGcounter) And rawAD < 255 Then
            rawAD = rawAD + 1
        End If

        chData(ch).ADvalue = rawAD
        Dim rawVoltage As Double = (5 / 255) * rawAD

        chData(ch).voltsCur = Format(chData(ch).convertValue(rawVoltage), "0.000")
        If chData(ch).voltsCur > chData(ch).voltsMax Then chData(ch).voltsMax = chData(ch).voltsCur
        If chData(ch).voltsCur < chData(ch).voltsMin Or chData(ch).voltsMin = -1 Then chData(ch).voltsMin = chData(ch).voltsCur

        chData(ch).rawAVGtotal = 0
        chData(ch).AVGcounter = 0
    End Sub



    ''' <summary>
    ''' writeLogLine() will append a line to the current CSV logfile.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub writeLogLine()
        If Not (sw Is Nothing) Then
            Try
                ' Determine the official list separator charactor that is used on the computer
                Dim listSeparator As String = System.Globalization.CultureInfo.CurrentCulture.TextInfo.ListSeparator

                ' Verify if the user tried to insert a text
                Dim textToAdd As String = ""
                If textToInsert <> "" Then
                    textToAdd = textToInsert
                    textToInsert = ""
                End If

                ' Calculate the number of milliseconds since the start of the log
                Dim elapsedTicks As Int64 = Int((Now.Ticks - startTicks) / 10000)

                ' Start building the CSV line.
                Dim csvLine As String
                csvLine = delimitIfNecessary(Now.ToString) & listSeparator & _
                          delimitIfNecessary(elapsedTicks) & listSeparator

                ' Check what data can be added
                For i As Integer = 0 To 1
                    If chData(i).enable = True Then
                        ' Only add this data when CH1 is enabled.
                        csvLine += delimitIfNecessary(chData(i).ADvalue) & listSeparator & _
                              delimitIfNecessary(chData(i).voltsCur) & listSeparator
                    End If
                Next

                ' Add optional commment data
                csvLine += textToAdd

                ' write the data to the file
                sw.WriteLine(csvLine)

                ' Update the information on the screen
                lineCount = lineCount + 1
                lblLineCount.Text = "Datalog size = " & lineCount & " records"
                lblDuration.Text = "Logging time = " & (elapsedTicks / 1000) & " seconds"


            Catch ex As Exception
                messageBox("An error occured during the logging. Logging will be stopped:" & vbCrLf & ex.Message)
                dataloggerRunning = False
            End Try
        End If
    End Sub


    ''' <summary>
    ''' tmrBgReader_Tick will be launched periodically to read the A/D channels of the K8055
    ''' card. This event is executed in a background thread.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tmrBgReader_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrBgReader.Tick

        Dim sampleTaken As Boolean = False

        ' Sample the two A/D channels
        For i = 1 To 2
            If sampleChannel(i) Then
                ' The required number of samples for the averaging are taken
                calculateStatistics(i)

                ' Show the info on the screen
                If i = 1 Then
                    vm1.Text = chData(i - 1).voltsCur & " V"
                    lblADC1.Text = "ADC = " & chData(i - 1).ADvalue & " steps"
                    lblMax1.Text = "Max = " & chData(i - 1).voltsMax & " V"
                    lblMin1.Text = "Min = " & chData(i - 1).voltsMin & " V"
                Else
                    vm2.Text = chData(i - 1).voltsCur & " V"
                    lblADC2.Text = "ADC = " & chData(i - 1).ADvalue & " steps"
                    lblMax2.Text = "Max = " & chData(i - 1).voltsMax & " V"
                    lblMin2.Text = "Min = " & chData(i - 1).voltsMin & " V"
                End If

                ' Invert the circle on the GUI
                If i = 1 Then invertShape()

                ' Set the sampleTaken flag
                sampleTaken = True
            End If
        Next


        ' Write the data to the datalogger
        If dataloggerRunning = True And sampleTaken = True Then
            writeLogLine()
        End If
    End Sub



    ''' <summary>
    ''' writeToLog will prepare a String that is to be inserted in the datalog: this String
    ''' will be put in the textToInsert variable. Afterwards the focus is given to the
    ''' inpMarker control.
    ''' </summary>
    ''' <param name="text"></param>
    ''' <remarks></remarks>
    Public Sub writeToLog(ByVal text As String)
        If dataloggerRunning = True Then
            ' Add to the log
            textToInsert = text
        Else
            messageBox("Cannot insert data to the log: the datalogger is currently not recording")
            inpMarker.Select()
        End If
    End Sub



    ''' <summary>
    ''' cmdRecord_Click is launched when the user presses the 'Record' button.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdRecord_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdRecord.Click

        If dataloggerRunning = False Then
            If myK8055.isConnected Then
                ' K8055 is connected: start the recorder procedure

                If chData(0).enable = False And chData(1).enable = False Then
                    messageBox("There are no measuring channels active. You need at least one active channel")
                    Exit Sub
                Else
                    startRecorderDialog()
                End If
            Else
                ' K8055 is not found: show an error
                messageBox("Error: the K8055 card is not connected")
            End If
        Else
            ' Datalogger is not running
            messageBox("Attention: the datalogger is already running")


        End If
    End Sub




    ''' <summary>
    ''' startRecorderDialog will show the file dialog to specify the location of the 
    ''' logged file.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub startRecorderDialog()
        With DlgSaveFile
            .Title = "Select a place to store your log file"
            .Filter = "(CSV Files)|*.csv"
            .FileName = "*.csv"
            .OverwritePrompt = True

            Dim res As DialogResult
            res = .ShowDialog

            If res = Windows.Forms.DialogResult.OK Then
                dataloggerFile = .FileName
                dataloggerRunning = True    ' we start logging data
                dataloggerStartTime = Now
                sw = New StreamWriter(.FileName)
                textToInsert = ""
                startTicks = Now.Ticks
                lineCount = 0

                Dim listSeparator As String = System.Globalization.CultureInfo.CurrentCulture.TextInfo.ListSeparator
                Dim csvLine As String

                csvLine = delimitIfNecessary("TIME") & listSeparator & _
                          delimitIfNecessary("TIME DIFF (ms)") & listSeparator

                If chData(0).enable Then
                    csvLine += delimitIfNecessary("A/D VALUE 1 (" & chData(0).name & ")") & listSeparator & _
                               delimitIfNecessary("A/D VOLTAGE 1 (" & chData(0).name & ")") & listSeparator
                End If

                If chData(1).enable Then
                    csvLine += delimitIfNecessary("A/D VALUE 2 (" & chData(0).name & ")") & listSeparator & _
                               delimitIfNecessary("A/D VOLTAGE 2 (" & chData(0).name & ")") & listSeparator
                End If

                csvLine += delimitIfNecessary("REMARK")
                sw.WriteLine(csvLine)
            End If
        End With
    End Sub
#End Region





#Region "Other Form Events"
    ''' <summary>
    ''' cmdConnect_Click will connect/disconnect to the K8055 card. 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdConnect.Click
        If cmdConnect.Text = "Connect" Then
            createConnection()
        Else
            closeConnection()
        End If
    End Sub



    ''' <summary>
    ''' cmdSetCh_LinkClicked will launch the dialog box for the required channel
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdSetCh_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles cmdSetCh1.LinkClicked,
                                                                                                                                        cmdSetCh2.LinkClicked
        Dim frm As New frmChanConfig

        If dataloggerRunning = True Then
            messageBox("Editing a channel is not possible while the datalogger is recording")

        Else
            ' Find out which button was pressed and load the corresponding channel data in the form
            If sender.Equals(cmdSetCh1) Then
                frm.currentConfig = chData(0)
            Else
                frm.currentConfig = chData(1)
            End If

            If frm.ShowDialog() = DialogResult.OK Then
                ' Find out which button was pressed and load the corresponding channel data in the form
                groupCH1.Text = "CH 1 - " & chData(0).name
                groupCH2.Text = "CH 2 - " & chData(1).name

                If chData(0).enable = False Then vm1.Text = "OFF"
                If chData(1).enable = False Then vm2.Text = "OFF"
            End If
        End If
    End Sub



    ''' <summary>
    ''' resetMin2_LinkClicked will reset the data for channel 2
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub resetMin2_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles resetMin2.LinkClicked
        With chData(1)
            .voltsMin = -1
            .voltsMax = 0
        End With
    End Sub


    ''' <summary>
    ''' resetMin1_LinkClicked will reset the data for channel 1
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub resetMin1_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles resetMin1.LinkClicked
        With chData(0)
            .voltsMin = -1
            .voltsMax = 0
        End With
    End Sub



    ''' <summary>
    ''' cmdStop_Click is launched when the user presses the STOP button. It only has effect when there
    ''' is a recording in progress.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStop.Click
        If dataloggerRunning Then
            If Not sw Is Nothing Then
                dataloggerRunning = False   ' we stop logging data
                sw.Close()
                sw = Nothing

                Dim ans As Integer
                ans = MsgBox("Do you want to open the saved CSV file with its default application?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.DefaultButton3 + MsgBoxStyle.Question, "Open CSV file?")

                If ans = MsgBoxResult.Yes Then
                    ' here we will open the current file
                    openFileWithDefaultProgram(dataloggerFile)
                End If
            End If
        Else
            messageBox("The datalogger is currently not running")
        End If
    End Sub


    ''' <summary>
    ''' cmbOptions_SelectedIndexChanged will be lanched when the user selects another option for
    ''' the logging interval.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmbOptions_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbOptions.SelectedIndexChanged
        lblInterval.Text = "Reading Interval = " & ((cmbOptions.SelectedIndex + 1) * 50) & " ms"
    End Sub


    ''' <summary>
    ''' inpMarker_KeyDown is launched when the user presses a key when the inpMarker textbox has the focus.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub inpMarker_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles inpMarker.KeyDown
        If e.KeyCode = Keys.Enter Then
            ' Catch the enter keypress and write the data to the log
            writeToLog(inpMarker.Text)
        End If
    End Sub


    ''' <summary>
    ''' cmdInsert_Click handles the insertion button: it will cause a call to the writeToLog() procedure.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdInsert_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdInsert.Click
        writeToLog(inpMarker.Text)
    End Sub
#End Region




#Region "Helper Functions"
    ''' <summary>
    ''' invertShape will invert the color of the red circle that indicates the update speed of the
    ''' voltage logger (selected interval).
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub invertShape()
        If shape.FillColor = Color.Red Then
            shape.FillColor = Color.Gray
        Else
            shape.FillColor = Color.Red
        End If
    End Sub


    ''' <summary>
    ''' The sub openFileWithDefaultProgram will open the specified file with its default application.
    ''' CSV files are usually set to be opened with Microsoft Excel.
    ''' </summary>
    ''' <param name="file"></param>
    ''' <remarks></remarks>
    Public Sub openFileWithDefaultProgram(ByVal file As String)
        Try
            System.Diagnostics.Process.Start(file)
        Catch ex As Exception
            MsgBox("An error occured: " & ex.Message, _
                    MsgBoxStyle.OkOnly + MsgBoxStyle.Critical, _
                    "Error while opening the process")
        End Try
    End Sub





    ''' <summary>
    ''' delimitIfNecessary will build a delimited variable that follows the guidlines
    ''' for the .CSV format.
    ''' </summary>
    ''' <param name="text">The variable to be delimited.</param>
    ''' <returns>The delimited result.</returns>
    ''' <remarks></remarks>
    Private Function delimitIfNecessary(ByVal text As String) As String
        text = text.Replace(Chr(10), "")
        text = text.Replace(Chr(13), "")

        ' Check 1: case of a nothing String 
        If text Is Nothing Then text = ""

        ' Added: fix to the bug !
        text = text.Replace("""", """""")

        ' Check 2: place between brackets if the text contains the " character, or a comma
        If text.Contains("""") Or text.Contains(",") Then
            Return """" & text & """"
        End If

        ' Check 3: place the text between brackets if the string has trailing or leading spaces
        If text.Trim() <> text Then
            Return """" & text & """"
        Else
            Return text
        End If

    End Function
#End Region




#Region "K8055 coding"
    ''' <summary>
    ''' createConnection() establishes a connection with the K8055 card
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub createConnection()
        ' Try to connect through the myK8055 object
        myK8055.connect(chkSK5.Checked, chkSK6.Checked)

        ' Evaluate the result...
        If myK8055.isConnected Then
            tmrBgReader.Enabled = True
            cmdConnect.Text = "Disconnect"
            REQ_SAMPLES = (cmbOptions.SelectedIndex + 1)
            chData(0).min = -1
            chData(1).min = -1
            chData(1).max = 0
            chData(0).max = 0
            cmbOptions.Enabled = False
            chkSK5.Enabled = False
            chkSK6.Enabled = False
        Else
            cmbOptions.Enabled = True
            shape.FillColor = Color.Gray
            chkSK5.Enabled = True
            chkSK6.Enabled = True
            messageBox("Error: K8055 connection failed (card not found? Select jumper configuration)")
        End If
    End Sub



    ''' <summary>
    ''' closeConnection() closes the connection with the K8055 card
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub closeConnection()
        myK8055.disconnect()
        tmrBgReader.Enabled = False
        cmbOptions.Enabled = True
        lblADC1.Text = "OFF"
        lblADC2.Text = "OFF"
        vm1.Text = "OFF"
        vm2.Text = "OFF"
        chkSK5.Enabled = True
        chkSK6.Enabled = True
        cmdConnect.Text = "Connect"
        shape.FillColor = Color.Gray
    End Sub
#End Region
End Class